LinkedList

Struct LinkedList 

pub struct LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,
{ /* private fields */ }
A LinkedList implementation that mimics std::map behavior from C++

  • Supports bidirectional iteration
  • Uses only the less-than operation for comparisons
  • Provides hint-based insertion

Implementations

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn contains_key(&self, key: &K) -> bool

Check if a key exists in the LinkedList

pub fn get(&self, search_key: &K) -> Option<&V>

Get a reference to the value associated with a key

pub fn get_at(&self, index: Index) -> Option<(&K, &V)>

Get a reference to the key and value at a given node index

pub fn get_k_at(&self, index: Index) -> Option<&K>

Get a reference to the key and value at a given node index

pub fn get_v_at(&self, index: Index) -> Option<&V>

Get a reference to the key and value at a given node index

pub fn get_kv(&self, search_key: &K) -> Option<(&K, &V)>

Get a reference to the value associated with a key

pub fn get_mut(&mut self, search_key: &K) -> Option<&mut V>

Get a mutable reference to the value associated with a key

pub fn set_v_at(&mut self, index: Index, input_val: V) -> Option<V>

Get a reference to the key and value at a given node index

pub fn get_mut_at(&mut self, index: Index) -> Option<(&K, &mut V)>

Get a mutable reference to the value at a given node index

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn insert_with_hint( &mut self, key: K, value: V, hint: Index, ) -> Result<Index, CppMapError>

Insert a key-value pair at the specified pos if possible. Returns the index of the inserted node Does nothing if that exact key already exists in the list

pub fn insert(&mut self, key: K, value: V) -> Result<Index, CppMapError>

Insert a key-value pair Returns the index of the inserted node Does nothing if that exact key already exists in the list

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn iter(&self) -> LinkedListIter<'_, K, V>

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn len(&self) -> usize

Get the number of elements in the LinkedList

pub fn is_empty(&self) -> bool

Check if the LinkedList is empty

pub fn sequential_find_position( &self, key: &K, hint_position: Index, ) -> Option<Index>

pub fn lower_bound(&self, key: &K) -> Option<Index>

Emulates the C++ std::map.lower_bound() Returns a cursor pointing at the first element that is greater or equal than the given key.

pub fn upper_bound(&self, key: &K) -> Option<Index>

Emulates the C++ std::map.upper_bound() Returns a cursor pointing at the first element in the map whose key is greater than the given key

pub fn next_pos(&self, current: Option<Index>) -> Option<Index>

Get the next node index (for iterator-like traversal)

pub fn prev_pos(&self, current: Option<Index>) -> Option<Index>

Get the previous node index (for bidirectional iterator-like traversal)

pub fn is_pos_valid(&self, index: Option<Index>) -> bool

pub fn first(&self) -> Option<Index>

Get the first element (equivalent to begin() in std::map)

pub fn last(&self) -> Option<Index>

Get the last element (equivalent to rbegin() in std::map)

pub fn is_at_first(&self, pos: Option<Index>) -> bool

Return true if pointer is at head position or if the list is empty

pub fn is_at_last(&self, pos: Option<Index>) -> bool

Return true if pointer is at tail position or if the list is empty

pub fn change_key_of_node( &mut self, index: Index, new_key: K, ) -> Result<Index, CppMapError>

Danger zone: replace the key of a specific node. This is only supposed to be done on keys that evaluates as identical

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn remove_by_index( &mut self, cursor_index: &mut Option<Index>, ) -> Option<(K, V)>

Removes node at current index, updating the index to point to the next valid position Returns the key-value pair if removal was successful. Moves cursor current to the old prev value if existed. Else pick old next index.

pub fn remove(&mut self, key: &K) -> Option<(K, V)>

Remove a node by key Returns the value of the removed node if found

pub fn clear(&mut self)

Clear the LinkedList

Source

impl<K, V> LinkedList<K, V>
where K: Debug + Display + Clone + IsLessThan, V: Debug + Clone,

pub fn validate(&self)

pub fn assert_is_in_order(&self)
where K: Debug + Clone + IsLessThan, V: Debug + Clone,

pub fn debug_print(&self)
where K: Display,

Print the LinkedList in a visual tabulated format for debugging